/*
 * $Id: MainProcess.java,v 1.4 2003/01/30 15:47:46 triolet Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 * Copyright (C) INRIA Rhne-Alpes, 2003.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package fr.fluxmedia.transmorpher.graph;

import fr.fluxmedia.transmorpher.engine.TApplyProcess;
import fr.fluxmedia.transmorpher.engine.TMain;
import fr.fluxmedia.transmorpher.engine.TProcess;
import fr.fluxmedia.transmorpher.engine.TProcessComposite;
import fr.fluxmedia.transmorpher.utils.LinearIndexedStruct;
import fr.fluxmedia.transmorpher.utils.Version;
import fr.fluxmedia.transmorpher.utils.Writer;

import java.io.IOException;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.ListIterator;
import java.util.Stack;

/**
 * The MainProcess class is an helper class which provides methods to create an execution structure
 * and methods to produce the java code corresponding to its type.
 *
 * MainProcess can be :
 * <ul>
 * <li>a Main, which can be executed or compiled</li>
 * <li>a Servlet, which can only be compiled</li>
 * <li>a Transformer, which can only be compiled</li>
 * </ul>
 *  
 *@author    Jerome.Euzenat@inrialpes.fr
 *@since     jdk 1.3 / SAX 2.0
 */
public abstract class MainProcess extends fr.fluxmedia.transmorpher.graph.Process {

	/**
	 * Constructor for the MainProcess object. 
	 *
	 *@param  name    The name of the MainProcess instance
	 *@param  owner   The transmorpher that owns this instance
	 *@param  nbIn		The number of in ports of this instance (should be 0)
	 *@param  nbOut		The number of out ports of this instance (should be 0)
	 */
	public MainProcess(String name, Transmorpher owner, int nbIn, int nbOut) {
		super(name, owner , nbIn, nbOut);
	}

	/**
	 *  Create an execution components that can contain others
	 *
	 *@return    Description of the Return Value
	 */
	public abstract TProcessComposite createMainProcess();

	/**
	 *  Prints a java code description of this MainProcess instance
	 *
	 *@param  file            The writer used for printing in a file
	 *@param  externs         Description of the Parameter
	 *@exception  IOException When IO errors occur
	 */
	public void generateJavaCode(Writer file, LinearIndexedStruct externs) throws IOException {
		file.writeln("/** Generated by " + Version.NAME + " " + Version.RELEASE);
		file.writeln("    See " + Version.URL);
		Date date = new Date();
		file.writeln("    " + date + " **/");
		file.writeln();
		file.writeln("// import Java classes");
		file.writeln("import java.util.Stack;");
		file.writeln();
		generateImport(file, externs);
		file.writeln("// import transmorpher classes");
		file.writeln("import fr.fluxmedia.transmorpher.engine.*;");
		file.writeln("import fr.fluxmedia.transmorpher.utils.*;");
		file.writeln();
		generateClassHeader(file);
		file.writeln(4, " // ressource manager");
		//iFile.writeln(4,"static SystemResources vSR = null;");
		file.writeln();
		file.writeln(4, "//global variables for port");
		file.writeln(4, "static String[] portIn  = null ;");
		file.writeln(4, "static String[] portOut = null;");
		file.writeln(4, "static String[] LocIn  = null ;");
		file.writeln(4, "static String[] LocOut = null;");
		file.writeln();
		file.writeln(4, " // global variable for the processFactory");
		file.writeln();
		file.writeln(4, " // stack of composite processes");
		file.writeln(4, "static Stack tmProcessStack = null;");
		file.writeln();
		file.writeln(4, " // global variables for attributes and parameters");
		// Default values should be that of Transmorpher
		file.writeln(4, "static int debug_mode = 0;");
		file.writeln(4, "static Parameters tmParameters = null ;");
		file.writeln(4, "static StringParameters tmAttributes = null;");
		file.writeln();
		generateMainHeader(file);
		//temporaire
		file.writeln(4, "double begin = System.currentTimeMillis();");
		file.writeln(4, "double end = 0;");
		file.writeln(4, "TProcessComposite tmCurrentCompositeProcess = null;");
		file.writeln(4, "TProcess tmCurrentProcess = null;");
		generateReadParameters(file);
		generateBody(file, externs);
		generateEnd(file);
	}

	/**
	 *  Generates java code for import
	 *
	 *@param  file            The writer used for printing in a file
	 *@param  externs         Description of the Parameter
	 *@exception  IOException When IO errors occur
	 */
	public abstract void generateImport(Writer file, LinearIndexedStruct externs) throws IOException;

	/**
	  *  Generates java code for class header
	 *
	 *@param  file            The writer used for printing in a file
	 *@exception  IOException When IO errors occur
	 */
	public abstract void generateClassHeader(Writer file) throws IOException;

	/**
	 *  Generates java code for main header
	 *
	 *@param  file            The writer used for printing in a file
	 *@exception  IOException When IO errors occur
	 */
	public abstract void generateMainHeader(Writer file) throws IOException;

	/**
	*  Generates java code for commmand line parameters
	 *
	 *@param  file            The writer used for printing in a file
	 *@exception  IOException When IO errors occur
	 */
	public abstract void generateReadParameters(Writer file) throws IOException;

	/**
	 *  Generates java code for body
	 *
	 *@param  file            The writer used for printing in a file
	 *@param  externs          Description of the Parameter
	 *@exception  IOException When IO errors occur
	 */
	public abstract void generateBody(Writer file, LinearIndexedStruct externs) throws IOException;

	/**
	 *  Generates java code for end
	 *
	 *@param  file            The writer used for printing in a file
	 *@exception  IOException When IO errors occur
	 */
	public abstract void generateEnd(Writer file) throws IOException;

}

